\name{compile}
\alias{compile}
\title{Byte Code Compiler}
\usage{
cmpfun(f, env = NULL)
cmpfile(infile, outfile, ascii = FALSE)
compile(e, env=NULL, promise = FALSE)
loadcmp(file, envir = .GlobalEnv, chdir = FALSE) 
disassemble(code)
}
\alias{cmpfun}
\alias{cmpfile}
\alias{compile}
\alias{loadcmp}
\alias{disassemble}
\arguments{
  \item{f}{a closure.}
  \item{env}{a static environment description; not yet useful at top level.}
  \item{file,infile,outfile}{pathnames; outfile defaults to infile with a .Rc
                             extension in place of any existing extension.}
  \item{ascii}{logical; should the compiled file be saved with in ascii
               format?}
  \item{promise}{logical; not yet useful at top level.}
  \item{envir}{environment to evaluate loaded expressions in.}
  \item{chdir}{logical; change directory before evaluation?}
  \item{code}{byte code expression or compiled closure}
  \item{e}{expression to compile}
}
\description{
  These functions provide an interface to a byte code compiler for R.
}
\details{
  The function \code{cmpfun} compiles the body of a closure and
  returns a new closure with the same formals and the body replaced by
  the compiled body expression.

  \code{cmpfile} parses the expression in \code{infile}, compiles
  them, and writes the compiled expressions to \code{outfile}.  If
  \code{outfile} is not provided, it is formed from \code{infile} by
  replacing or appending a \code{.Rc} suffix.

  \code{compile} compiles an expression into a byte code object; the
  object can then be evaluated with \code{eval}.  \code{compile} is mainly
  used inside the compiler itself; it might be useful on its own, but
  I'm not sure.  It may be removed from the public interface

  \code{loadcmp} is used to load compiled files.  It is similar to
  \code{sys.source}, except that its default loading environment is the
  global environment rather than the base environment.

  \code{disassemble} produces a printed representation of the code
  that may be useful to give a hint of what is going on.

  Currently the compiler warns about a variety of things.  It does
  this by using \code{cat} to print messages.  Eventually this should
  use the condition handling mechanism.

  The compiler does not make use of the name space mechanism yet.
  It assumes that global functions with certain names are the ones
  defined in base.  This can cause trouble if code is intended to use
  a finction defined in a package or the global environment that
  shadows one defined in base.

  Calling the compiler a byte code compiler is actually a bit of a
  misnomer: the external representation of code objects currently uses
  \code{int} operands, and when compiled with \code{gcc} the internal
  representation is actually threaded code rather than byte code.
}
\author{Luke Tierney}

\examples{
# a simple example
f <- function(x) x+1
fc <- cmpfun(f)
fc(2)
disassemble(fc)

# old R version of lapply
la1 <- function(X, FUN, ...) {
    FUN <- match.fun(FUN)
    if (!is.list(X))
	X <- as.list(X)
    rval <- vector("list", length(X))
    for(i in seq(along = X))
	rval[i] <- list(FUN(X[[i]], ...))
    names(rval) <- names(X)		  # keep `names' !
    return(rval)
}
# a small variation
la2 <- function(X, FUN, ...) {
    FUN <- match.fun(FUN)
    if (!is.list(X))
	X <- as.list(X)
    rval <- vector("list", length(X))
    for(i in seq(along = X)) {
        v <- FUN(X[[i]], ...)
        if (is.null(v)) rval[i] <- list(v)
        else rval[[i]] <- v
    }
    names(rval) <- names(X)		  # keep `names' !
    return(rval)
}
# Compiled versions
la1c <- cmpfun(la1)
la2c <- cmpfun(la2)
# some timings
x<-1:10
y<-1:100
system.time(for (i in 1:10000) lapply(x, is.null))
system.time(for (i in 1:10000) la1(x, is.null))
system.time(for (i in 1:10000) la1c(x, is.null))
system.time(for (i in 1:10000) la2(x, is.null))
system.time(for (i in 1:10000) la2c(x, is.null))
system.time(for (i in 1:1000) lapply(y, is.null))
system.time(for (i in 1:1000) la1(y, is.null))
system.time(for (i in 1:1000) la1c(y, is.null))
system.time(for (i in 1:1000) la2(y, is.null))
system.time(for (i in 1:1000) la2c(y, is.null))
}
\keyword{programming}
